#!/usr/bin/env python3
 
#Author: Erik Bergstrom

#Contact: ebergstr@eng.ucsd.edu

import os
import re
import sys
import argparse


def save_chrom_strings (genome):
	current_dir = os.getcwd()
	ref_dir = re.sub('\/scripts$', '', current_dir)
	chrom_fasta_path = ref_dir + '/references/chromosomes/fasta/' + genome + "/"
	chrom_string_path = ref_dir + '/references/chromosomes/chrom_string/' + genome + '/'
	chrom_fasta_files = os.listdir(chrom_fasta_path)

	if not os.path.exists(chrom_string_path):
		os.makedirs(chrom_string_path)

	first = True
	i = 1
	if ".DS_Store" in chrom_fasta_files:
		chrom_fasta_files.remove('.DS_Store')
	chrom_total = len(chrom_fasta_files)
	for files in chrom_fasta_files:
		if not files.startswith('.'):
			file_name = files.split(".")
			try:
				chromosome = file_name[-2]
			except:
				continue
			if file_name[-4] == 'dna':
				with open(chrom_fasta_path + files) as chrom, open(chrom_string_path + chromosome + ".txt", 'w') as out:
					next(chrom)
					chromosome_final = ''
					for lines in chrom:
						line = lines.strip()
						chromosome_final += line


					print(chromosome_final, file=out)
					if first:
						print("The string file has been created for Chromosome: " + chromosome + " (" + str(i) + "/" + str(chrom_total) + ")")
						first = False
					else:
						print("                                                 " + chromosome + " (" + str(i) + "/" + str(chrom_total) + ")")
		i += 1

def main():
	parser = argparse.ArgumentParser(description="Provide the necessary arguments to save the chromosomes as strings for later reference.")
	parser.add_argument("--genome", "-g",help="Provide a reference genome. (ex: GRCh37, GRCh38, mm10)")
	args=parser.parse_args()
	genome = args.genome

	save_chrom_strings(genome)


if __name__ == '__main__':
	main()


#!/usr/bin/env python3

#Author: Erik Bergstrom

#Contact: ebergstr@eng.ucsd.edu

import os
import sys
import re
import subprocess
import argparse
import time
from scipy import spatial
import pandas as pd
import shutil
import logging
from SigProfilerMatrixGenerator.scripts import convert_input_to_simple_files as convertIn

def install_chromosomes (genomes, ref_dir, custom):
	if custom:
		for genome in genomes:
			os.system("gunzip references/chromosomes/fasta/" + genome + "/*.gz")
			chromosome_fasta_path = "references/chromosomes/fasta/" + genome + "/"
			os.system("python3 scripts/save_chrom_strings.py -g " + genome)
			print("Chromosome string files for " + genome + " have been created. Continuing with installation.")
			#os.system("rm -r " + chromosome_fasta_path)
	else:

		for genome in genomes:
			species = None
			chrom_number = None
			if genome == 'GRCh37' or genome == 'GRCh38': 
				species = "homo_sapiens"
				chrom_number = 24
			elif genome == 'mm10' or genome == 'mm9':
				species = "mus_musculus"
				chrom_number = 21
			else:
				print(genome + " is not supported. The following genomes are supported:\nGRCh37, GRCh38, mm10")
			
			chromosome_string_path = "references/chromosomes/chrom_string/" + genome + "/"
			chromosome_fasta_path = "references/chromosomes/fasta/" + genome + "/"

			wget_flag = True
			if os.path.exists(chromosome_string_path) == False or len(os.listdir(chromosome_string_path)) <= chrom_number:
				if os.path.exists(chromosome_fasta_path) == False or len(os.listdir(chromosome_fasta_path)) <= chrom_number:
					print("Chromosomes are not currently saved as individual text files for " + genome + ". Downloading the files now...")
					try:
						p = subprocess.Popen("wget", stdout=subprocess.PIPE, stderr=subprocess.PIPE)
					except:
						proceed = input("You may not have wget or homebrew installed. Download those dependencies now?[Y/N]").upper()
						if proceed == 'Y':
							try:
								os.system("brew install wget")
							except:
								os.system('/usr/bin/ruby -e "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/master/install)"')
								os.system("brew install wget")
						else:
							print("Installation has stopped. Please download the chromosome files before proceeding with the installation.")
							wget_flag = False
							sys.exit()
					if wget_flag:
						try:
							if genome == 'GRCh37':
								os.system("wget -r -l1 -c -nc --no-parent -A '*.dna.chromosome.*' -nd -P " + chromosome_fasta_path + " ftp://ftp.ensembl.org/pub/grch37/update/fasta/homo_sapiens/dna/ 2>> install.log ")
							elif genome == 'mm9':
								os.system("wget -r -l1 -c -nc --no-parent -A '*.dna.chromosome.*' -nd -P " + chromosome_fasta_path + " ftp://ftp.ensembl.org/pub/release-67/fasta/mus_musculus/dna/ 2>> install.log")
							else:
								os.system("wget -r -l1 -c -nc --no-parent -A '*.dna.chromosome.*' -nd -P " + chromosome_fasta_path + " ftp://ftp.ensembl.org/pub/release-93/fasta/"+species+"/dna/ 2>> install.log")
							os.system("gunzip references/chromosomes/fasta/" + genome + "/*.gz")
						except:
							print("The ensembl ftp site is not currently responding.")
							sys.exit()

				print("Chromosome fasta files for " + genome + " have been installed. Creating the chromosome string files now...")
				os.system("python3 scripts/save_chrom_strings.py -g " + genome)
				print("Chromosome string files for " + genome + " have been created. Continuing with installation.")
				os.system("rm -r " + chromosome_fasta_path)

			else:
				print("Chromosome reference files exist for " + genome + ". Continuing with installation.")


def install_chromosomes_tsb (genomes, ref_dir, custom):
	for genome in genomes:
		chrom_string_path = "references/chromosomes/chrom_string/" + genome + "/"
		chrom_number = len(chrom_string_path)

		chromosome_TSB_path = "references/chromosomes/tsb/" + genome + "/"
		transcript_files = "references/chromosomes/transcripts/" + genome + "/"

		if os.path.exists(transcript_files) == False or len(os.listdir(transcript_files)) < 1:
			print("Please download the transcript files before proceeding. You can download the files from 'http://www.ensembl.org/biomart/martview'.")
			print("Follow the format presented in the README file:\n\n\tGene stable ID  Transcript stable ID    Chromosome/scaffold name    Strand  Transcript start (bp)   Transcript end (bp)\n\n\n")
			sys.exit()
		if os.path.exists(chromosome_TSB_path) == False or len(os.listdir(chromosome_TSB_path)) < chrom_number:
			print("The transcriptional reference data for " + genome + " has not been saved. Creating these files now")
			os.system("python3 scripts/save_tsb_192.py -g " + genome)

		print("The transcriptional reference data for " + genome + " has been saved.")

def install_chromosomes_tsb_BED (genomes, custom, ref_dir):
	for genome in genomes:
		if not os.path.exists(ref_dir + "chromosomes/tsb_BED/" + genome + "/") or len(os.listdir(ref_dir + "chromosomes/tsb_BED/" + genome + "/")) < 19:
			os.system("python3 scripts/save_chrom_tsb_separate.py -g " + genome)
			print("The TSB BED files for " + genome + " have been saved.")

def benchmark (ref_dir):
	if os.path.exists("scripts/Benchmark/BRCA_bench/"):
		shutil.move("scripts/Benchmark/BRCA_bench/", "references/vcf_files/")
	current_dir = os.path.realpath(__file__)
	ref_dir = re.sub('\/install.py$', '', current_dir)
	vcf_path = ref_dir + "/references/vcf_files/BRCA_bench/"
	output_path = ref_dir + "/references/matrix/"

	start_time = time.time()
	os.system("python3 " + ref_dir + "/scripts/sigProfilerMatrixGenerator.py -g GRCh37 -p BRCA_bench -vf " + vcf_path + " -op " + output_path + " -snv")
	end_time = time.time()

	original_matrix_96 = ref_dir + "/scripts/Benchmark/BRCA_bench_orig_96.txt"
	original_matrix_3072 = ref_dir + "/scripts/Benchmark/BRCA_bench_orig_3072.txt"
	new_matrix_96 = ref_dir + "/references/matrix/BRCA_bench/SBS/BRCA_bench.SBS96.all"
	new_matrix_3072 = ref_dir + "/references/matrix/BRCA_bench/SBS/BRCA_bench.SBS3072.all"

	genome = "GRCh37"

	############# Cosine Test ###################################################
	data_orig = pd.read_csv(original_matrix_96, sep='\t', header=0)
	data_new = pd.read_csv(new_matrix_96, sep='\t', header=0)
	count = 0
	range_count = min(len(data_orig.loc[0]), len(data_new.loc[0]))
	for i in range (1, range_count, 1):
	    orig_list = list(data_orig[data_orig.columns[i]])
	    new_list = list(data_new[data_new.columns[i]])
	    cosine_result = (1-spatial.distance.cosine(orig_list,new_list))
	    if cosine_result != 1:
	        count += 1
	if count != 0:
	    print("There seems to be some errors in the newly generated matrix. The installation may not have been successful.")


	data_orig = pd.read_csv(original_matrix_3072, sep='\t', header=0)
	data_new = pd.read_csv(new_matrix_3072, sep='\t', header=0)
	count = 0
	range_count = min(len(data_orig.loc[0]), len(data_new.loc[0]))
	for i in range (1, range_count, 1):
	    orig_list = data_orig[data_orig.columns[i]]
	    new_list = data_new[data_new.columns[i]]
	    cosine_result = (1-spatial.distance.cosine(orig_list,new_list))
	    if cosine_result <= 0.85:
	        count += 1
	if count != 0:
	    print("There seems to be some errors in the newly generated matrix. The installation may not have been successful.")

	end_time = time.time()
	print("Installation was succesful.\nSigProfilerMatrixGenerator took " + str(end_time-start_time) + " seconds to complete.")


def install (genome, custom=False):
	print("Beginning installation. This may take up to 20 minutes to complete.")
	first_path = os.getcwd()
	current_dir = os.path.realpath(__file__)
	ref_dir = re.sub('\/install.py$', '', current_dir)
	os.chdir(ref_dir)
	#genomes = ['mm9', 'mm10','GRCh37', 'GRCh38' ]
	#genomes = ['GRCh37']

	genomes = [genome]

	if os.path.exists("install.log"):
		os.system("rm install.log")

	ref_dir = "references/"
	chrom_string_dir = ref_dir + "chromosomes/chrom_string/"
	chrom_fasta_dir = ref_dir + "chromosomes/fasta/"
	chrom_tsb_dir = ref_dir + "chromosomes/tsb/"
	matrix_dir = ref_dir + "matrix/"
	vcf_dir = ref_dir + "vcf_files/"
	bed_dir = ref_dir + "vcf_files/BED/"
	log_dir = "logs/"
	new_dirs = [ref_dir, chrom_string_dir, chrom_fasta_dir, chrom_tsb_dir, matrix_dir, vcf_dir, bed_dir, log_dir]

	current_dir = os.getcwd()
	for dirs in new_dirs:
		if not os.path.exists(dirs):
			os.makedirs(dirs)

	install_chromosomes(genomes, ref_dir, custom)
	install_chromosomes_tsb (genomes, ref_dir, custom)

	if os.path.exists("BRCA_example/"):
		os.system("mv BRCA_example/ references/vcf_files/")
	if os.path.exists("example_test"):
		os.system("mv example_test/ references/vcf_files/")
	if os.path.exists("context_distributions/"):
		os.system("mv context_distributions/ references/chromosomes/")

	print("All reference files have been created.\nVerifying and benchmarking installation now...")
	if genome == "GRCh37":
		benchmark(ref_dir)
	# if os.path.exists(chrom_tsb_dir + "GRCh37/"):
	# 	benchmark(ref_dir)
	print ("To proceed with matrix_generation, please provide the path to your vcf files and an appropriate output path.")
	os.system("rm -r " + chrom_string_dir)
	print("Installation complete.")
	os.chdir(first_path)

def main ():

	first_path= os.getcwd()
	os.chdir(first_path + "/sigProfilerMatrixGenerator/")
	genomes = ['mm9', 'mm10','GRCh37', 'GRCh38' ]
	#genomes = ['GRCh37']
	custom = False
	parser = argparse.ArgumentParser(description="Provide the necessary arguments to install the reference files.")
	parser.add_argument("-g", "--genome", nargs='?', help="Optional parameter instructs script to install the custom genome.")
	parser.add_argument("-ct", "--custom", help="Optional parameter instructs script to create the reference files for a custom genome", action='store_true')
	args = parser.parse_args()

	if args.genome:
		genomes = [args.genome]
	if args.custom:
		custom = True

	if os.path.exists("install.log"):
		os.system("rm install.log")

	ref_dir = "references/"
	chrom_string_dir = ref_dir + "chromosomes/chrom_string/"
	chrom_fasta_dir = ref_dir + "chromosomes/fasta/"
	chrom_tsb_dir = ref_dir + "chromosomes/tsb/"
	matrix_dir = ref_dir + "matrix/"
	vcf_dir = ref_dir + "vcf_files/"
	bed_dir = ref_dir + "vcf_files/BED/"
	log_dir = "logs/"
	new_dirs = [ref_dir, chrom_string_dir, chrom_fasta_dir, chrom_tsb_dir, matrix_dir, vcf_dir, bed_dir, log_dir]

	current_dir = os.getcwd()
	for dirs in new_dirs:
		if not os.path.exists(dirs):
			os.makedirs(dirs)

	#initial_transcripts = os.listdir("transcripts_original/")
	#for file in initial_transcripts:
	#	name = file.split("_")
	#	if not os.path.exists("references/chromosomes/transcripts/"+name[0]+"/"):
	#		os.makedirs("references/chromosomes/transcripts/"+name[0]+"/")
	#	if name != ".DS":
	#		os.system("cp transcripts_original/"+file +" references/chromosomes/transcripts/" + name[0]+"/")

	#if os.path.exists("exome/"):
	#	os.system("mv exome/ references/chromosomes/")

	install_chromosomes(genomes, ref_dir, custom)
	install_chromosomes_tsb (genomes, ref_dir, custom)
	#install_chromosomes_tsb_BED (genomes, custom, ref_dir)
	if os.path.exists("BRCA_example/"):
		os.system("mv BRCA_example/ references/vcf_files/")
	if os.path.exists("example_test"):
		os.system("mv example_test/ references/vcf_files/")
	if os.path.exists("context_distributions/"):
		os.system("mv context_distributions/ references/chromosomes/")

	print("All reference files have been created.\nVerifying and benchmarking installation now...")
	if os.path.exists(chrom_tsb_dir + "GRCh37/"):
		benchmark(ref_dir)
	print ("Please place your vcf files for each sample into the 'references/vcf_files/[test]/[mutation_type]/' directory. Once you have done that, you can proceed with the matrix generation.")
	#os.system("rm -r " + chrom_string_dir)
	print("Installation complete.")
	os.chdir(first_path)

if __name__ == '__main__':
	main()


#!/usr/bin/env python3
 
#Author: Erik Bergstrom

#Contact: ebergstr@eng.ucsd.edu

from . import SigProfilerMatrixGenerator as matGen
import os
import re
import sys
import logging
import pandas as pd
import datetime
from SigProfilerMatrixGenerator.scripts import convert_input_to_simple_files as convertIn
import uuid
import shutil

def SigProfilerMatrixGeneratorFunc (project, genome, vcfFiles, exome=False, indel_extended=False, bed_file=None, chrom_based=False, plot=False, gs=False):
	'''
	Allows for the import of the sigProfilerMatrixGenerator.py function. Returns a dictionary
	with each context serving as the first level of keys. 

	Parameters:
			   project  -> unique name given to the current samples
				genome  -> reference genome 
				 exome  -> flag to use only the exome or not
				 indel  -> flag to create the matrix for the limited indel matrix
		indel_extended  -> flag to create the regular, extended INDEL matrix
			  bed_file  -> BED file that contains a list of ranges to be used in generating the matrices

	Returns:
			  matrices  -> dictionary (nested) of the matrices for each context

		example:
			matrices = {'96': {'PD1001a':{'A[A>C]A':23,
										 'A[A>G]A':10,...},
							  'PD1202a':{'A[A>C]A':23,
										 'A[A>G]A':10,...},...},
						'192':{'PD1001a':{'T:A[A>C]A':23,
										 'T:A[A>G]A':10,...},
							  'PD1202a':{'T:A[A>C]A':23,
										 'T:A[A>G]A':10,...},...},...}
	'''

	# Instantiates all of the required variables
	functionFlag = True
	bed = False
	bed_ranges = None
	# indel = indel
	if not indel_extended:
		limited_indel = True
	exome = exome
	plot = plot
	# if plot:
	# 	indel = True

	matrices = {'96':None, '1536':None, '192':None, '3072':None, 'DINUC':None, '6':None, '12':None, 'INDEL':None}

	# Adjusts the variables if the context is for INDELs
	# if indel:
	# 	matrices = {'INDEL':None}

	ncbi_chrom = {'NC_000067.6':'1', 'NC_000068.7':'2', 'NC_000069.6':'3', 'NC_000070.6':'4', 
				  'NC_000071.6':'5', 'NC_000072.6':'6', 'NC_000073.6':'7', 'NC_000074.6':'8',
				  'NC_000075.6':'9', 'NC_000076.6':'10', 'NC_000077.6':'11', 'NC_000078.6':'12',
				  'NC_000079.6':'13', 'NC_000080.6':'14', 'NC_000081.6':'15', 'NC_000082.6':'16', 
				  'NC_000083.6':'17', 'NC_000084.6':'18', 'NC_000085.6':'19', 'NC_000086.7':'X', 
				  'NC_000087.7':'Y'}
				  
	tsb_ref = {0:['N','A'], 1:['N','C'], 2:['N','G'], 3:['N','T'],
			   4:['T','A'], 5:['T','C'], 6:['T','G'], 7:['T','T'],
			   8:['U','A'], 9:['U','C'], 10:['U','G'], 11:['U','T'],
			   12:['B','A'], 13:['B','C'], 14:['B','G'], 15:['B','T'],
			   16:['N','N'], 17:['T','N'], 18:['U','N'], 19:['B','N']}


	contexts = ['3072', 'DINUC']


	# Organizes all of the reference directories for later reference:
	current_dir = os.path.realpath(__file__)

	ref_dir = re.sub('\/scripts/SigProfilerMatrixGeneratorFunc.py$', '', current_dir)
	#ref_dir = re.sub('\/scripts/SigProfilerMatrixGeneratorFunc_bi_new_structure.py$', '', current_dir)

	chrom_path =ref_dir + '/references/chromosomes/tsb/' + genome + "/"
	transcript_path = ref_dir + '/references/chromosomes/transcripts/' + genome + "/"


	if not os.path.exists(chrom_path):
		print("The specified genome: " + genome + " has not been installed\nRun the following command to install the genome:\n\tpython3 sigProfilerMatrixGenerator/install.py -g " + genome)
		sys.exit()

	# Organizes all of the input and output directories:
	#output_matrix = ref_dir + "/references/matrix/"
	#vcf_path = ref_dir + '/references/vcf_files/' + project + "/"


	if vcfFiles[-1] != "/":
		vcfFiles += "/"
	vcf_path = vcfFiles + "input/"
	vcf_path_original = vcf_path
	if not os.path.exists(vcf_path):
		os.mkdir(vcf_path)
		for files in os.listdir(vcfFiles):
			shutil.move(vcfFiles + files, vcf_path)
	output_matrix = vcfFiles + "output/"
	if not os.path.exists(output_matrix):
		os.system("mkdir " + output_matrix)

	# vcf_path = vcfFiles
	# if vcf_path[-1] != "/":
	# 	vcf_path += "/"
	# 	vcfFiles += "/" 
	# vcf_path += "input/"
	# vcf_path_original = vcf_path
	# output_matrix = vcfFiles
	# output_matrix += "output/"
	# if not os.path.exists(output_matrix):
	# 	os.system("mkdir " + output_matrix)


	time_stamp = datetime.date.today()
	#output_log_path = ref_dir + "/logs/"
	output_log_path = vcfFiles + "logs/"
	if not os.path.exists(output_log_path):
		os.system("mkdir " + output_log_path)
	error_file = output_log_path + 'SigProfilerMatrixGenerator_' + project + "_" + genome + str(time_stamp) + ".err"
	log_file = output_log_path + 'SigProfilerMatrixGenerator_' + project + "_" + genome + str(time_stamp) + ".out"
	if os.path.exists(error_file):
		os.system("rm " + error_file)
	if os.path.exists(log_file):
		 os.system("rm " + log_file)
	sys.stderr = open(error_file, 'w')
	logging.basicConfig(filename=log_file, level=logging.INFO, filemode='w')



	# Gathers all of the vcf files:
	vcf_files_temp = os.listdir(vcf_path)
	# for files in vcf_files_temp:
	# 	shutil.move(vcfFile + files, "input/")
	vcf_files = []
	first_extenstion = True
	for file in vcf_files_temp:
		# Skips hidden files
		if file[0:3] == '.DS' or file[0:2] == '__':
			pass
		else:
			vcf_files.append(file)

	# if SNVs:
	# 	vcf_files_snv_temp = os.listdir(vcf_path + "SNV/")
	# if indel:
	# 	vcf_files_indel_temp = os.listdir(vcf_path + "INDEL/")

	# vcf_files2 = [[],[]]

	# if SNVs:
	# 	for file in vcf_files_snv_temp:
	# 		# Skips hidden files
	# 		if file[0:3] == '.DS' or file[0:2] == '__':
	# 			pass
	# 		else:
	# 			vcf_files2[0].append(file)

	# if indel:
	# 	for file in vcf_files_indel_temp:
	# 		# Skips hidden files
	# 		if file[0:3] == '.DS' or file[0:2] == '__':
	# 			pass
	# 		else:
	# 			vcf_files2[1].append(file)


	# for i in range(0, len(vcf_files2), 1):
	# 	if i ==1 and indel:
	# 		contexts = ['INDEL']
	# 	elif i == 1 and not indel:
	# 		break
	# 	elif i == 0 and not SNVs:
	# 		continue
		#vcf_path = ref_dir + '/references/vcf_files/' + project + "/"
	file_name = vcf_files[0].split(".")
	file_extension = file_name[-1]

	unique_folder = project + "_"+ str(uuid.uuid4())
	output_path = output_matrix + "temp/" + unique_folder + "/"
	if os.path.exists(output_path):
		os.system("rm -r " + output_path)

	os.makedirs(output_path)

	if file_extension == 'genome':
			convertIn.convertTxt(project, vcf_path, genome, output_path)
	else:
		if file_extension == 'txt':
			snv, indel = convertIn.convertTxt(project, vcf_path,  genome,  output_path)
		elif file_extension == 'vcf':
			snv, indel = convertIn.convertVCF(project, vcf_path,  genome, output_path)
		elif file_extension == 'maf':
			snv, indel = convertIn.convertMAF(project, vcf_path,  genome, output_path)
		elif file_extension == '.tsv':
			snv, indel = convertIn.convertICGC(project, vcf_path,  genome, output_path)
		else:
			print("File format not supported")

################left off here with updates
	for i in range(0, 2, 1):
		if i == 0 and snv:
			output_path_snv = output_path + "SNV/"
			vcf_files = os.listdir(output_path_snv)
			vcf_path = output_path_snv
		elif i == 0 and not snv:
			continue
		elif i == 1 and indel:
			contexts = ['INDEL']
			output_path_indel = output_path + "INDEL/"
			vcf_files = os.listdir(output_path_indel)
			vcf_path = output_path_indel
		elif i ==1 and not indel:
			continue
		if ".DS_Store" in vcf_files:
			vcf_files.remove(".DS_Store")

		sort_file = vcf_files[0]
		with open(vcf_path + sort_file) as f:
			lines = [line.strip().split() for line in f]

		output = open(vcf_path + sort_file, 'w')

		try:
			for line in sorted(lines, key = lambda x: (['X','Y','1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', 'MT', 'M'].index(x[5]), x[1], x[6])):
				print('\t'.join(line), file=output)
		except ValueError as e:
			err = str(e).split("'")
			err = err[1]
			print("There appears to be unrecognized chromosomes within the file. Please remove the lines that contain the chromosome: " + err)
			sys.exit()
			
		output.close()

		print("Sorting complete...\nDetermining mutation type for each variant, one chromosome at a time. Starting catalogue generation...")

		if bed_file != None:
			bed = True
			bed_file_path = bed_file
			#bed_file_path = ref_dir + "/references/vcf_files/BED/" + project + "/" + bed_file
			bed_ranges = matGen.BED_filtering(bed_file_path)

		else:
			bed_file_path = None

		# Creates the matrix for each context
		for context in contexts:
			if context != 'DINUC' and context != 'INDEL':
				matrix = matGen.catalogue_generator_single (vcf_path, vcf_path_original, vcf_files, bed_file_path, chrom_path, project, output_matrix, context, exome, genome, ncbi_chrom, functionFlag, bed, bed_ranges, chrom_based, plot, tsb_ref, transcript_path, gs)
				matrices = matrix

			elif context == 'DINUC':
				matrix = matGen.catalogue_generator_DINUC_single (vcf_path, vcf_path_original, vcf_files, bed_file_path, chrom_path, project, output_matrix, exome, genome, ncbi_chrom, functionFlag, bed, bed_ranges, chrom_based, plot, tsb_ref)
				matrices[context] = matrix

			elif context == 'INDEL':
				matrix = matGen.catalogue_generator_INDEL_single (vcf_path, vcf_path_original, vcf_files, bed_file_path, chrom_path, project, output_matrix, exome, genome, ncbi_chrom, limited_indel, functionFlag, bed, bed_ranges, chrom_based, plot, tsb_ref, transcript_path, gs)
				matrices[context] = matrix

			# Deletes the temporary files and returns the final matrix
			logging.info("Catalogue for " + context + " context is complete.")
			print("Catalogue for " + context + " context is complete.")
		if i == 1:
			os.system("rm -r " + output_matrix + "temp/")

	final_matrices = {}
	for conts in matrices.keys():
		final_matrices[conts] = pd.DataFrame.from_dict(matrices[conts])
		for column in final_matrices[conts].columns:
			final_matrices[conts][column] = final_matrices[conts][column].fillna(0)
	return(final_matrices)

	


#!/usr/bin/env python3
 
#Author: Erik Bergstrom

#Contact: ebergstr@eng.ucsd.edu

import os
import sys
import time
import re
import argparse

start_time = time.time()

def save_tsb (chromosome_string_path, transcript_path, output_path):
    '''
    Creates binary files that contain the transcriptional information at a given 
    base. The transcriptional information entails Transcribed, Untranscribed, 
    Bi-directionally transcribed, and Non-transcribed. These files are required to
    simulate transcriptional strand bias.

    Input:
        chromosome_string_path -> path to chromosomes saved in individual text files
        transcript_path -> path to transcript data. The following should be used as the format
                           (additional information may be saved, however it will not
                           be used when creating the files):

        Gene stable ID  Transcript stable ID    Chromosome/scaffold name    Strand  Transcript start (bp)   Transcript end (bp)

        output_path -> path where the binary transcript files are saved.

    Output:
        Binary files that contain the transciptional information at each base for
        each chromosome. The files are saved as "[chrom]_192.txt"

    '''

    # Retrieves the transcript file 
    transcript_files = os.listdir(transcript_path)

    if len(transcript_files) < 3:
        # Instantiates required flag variables
        initial_flag = True
        initial_chrom = None
        for file in transcript_files:
            file_name = file.split("_")
            name = file_name[0]

            # Skips hidden files
            if name == '.DS':
                pass
            else:

                # Sorts the transcript file by chromosome and transcript range
                os.system("sort -t $'\t' -k 3,3n -k 3,3 -k 5,5 -k 6,6 " + transcript_path+file + " -o " + transcript_path+file)
                 
                with open (transcript_path + file) as f:
                    #next(f)

                    # Saves all transcriptional information for a chromosome into a unique file
                    for lines in f:
                        line = lines.strip()
                        line_split = line.split()
                        chrom = line_split[2]

                        if initial_flag:
                            initial_chrom = chrom
                            initial_flag = False
                            out = open (transcript_path+chrom + "_transcripts.txt", 'w')

                        if chrom == initial_chrom:
                            print(line, file=out)

                        else:
                            out.close()
                            initial_chrom = chrom
                            out = open(transcript_path+chrom+"_transcripts.txt",'w')
                    out.close()

            # Removes the initial transcript file
            os.system("rm " + transcript_path + file)

    # Retrieves the chromosome based list of transcript files 
    transcript_files = os.listdir(transcript_path)
    if '.DS_Store' in transcript_files:
        transcript_files.remove('.DS_Store')
    for files in transcript_files:
        with open(transcript_path + files) as f:
            lines = [line.strip().split() for line in f]

        output = open(transcript_path + files, 'w')

        for line in sorted(lines, key = lambda x: (int(x[4]))):
            print('\t'.join(line), file=output)

        output.close()

    print("Creating the transcriptional reference files now. This may take awhile...")

    first = True
    chrom_total = len(transcript_files)
    z = 1
    for files in transcript_files:
        file_name = files.split("_")
        chrom = file_name[0]
        
        if chrom == '.DS':
            pass
        else:
            try:
                if os.path.exists(output_path + chrom + ".txt"):
                    continue

                else:
                    # Create a binary file for the output.
                    outFile = open (output_path + chrom + ".txt", "wb")


                    # Instantiates all of the required parameters.
                    transcripts = None
                    bi_start = 0
                    bi_end= None
                    I_chrom = None
                    I_T_start = None
                    I_T_end = None
                    I_strand = None
                    location = 0
                    chrom_length = None

                    # Save the length for the current chromosome.
                    with open (chromosome_string_path + chrom + ".txt") as f:
                        chrom_string = f.readline().strip()
                        chrom_length = len(chrom_string)

                    # Establishes a unique binary number for each transcript type.
                    Non_transcribed_A = '00000000' # 0
                    Non_transcribed_C = '00000001' # 1
                    Non_transcribed_G = '00000010' # 2
                    Non_transcribed_T = '00000011' # 3
                    Transcribed_A = '00000100'     # 4
                    Transcribed_C = '00000101'     # 5
                    Transcribed_G = '00000110'     # 6
                    Transcribed_T = '00000111'     # 7
                    Untranscribed_A = '00001000'   # 8
                    Untranscribed_C = '00001001'   # 9
                    Untranscribed_G = '00001010'   # 10
                    Untranscribed_T = '00001011'   # 11
                    BiDirectional_A = '00001100'   # 12
                    BiDirectional_C = '00001101'   # 13
                    BiDirectional_G = '00001110'   # 14
                    BiDirectional_T = '00001111'   # 15
                    Non_transcribed_N = '00010000' # 16
                    Transcribed_N = '00010001'     # 17
                    Untranscribed_N = '00010010'   # 18 
                    BiDirectional_N = '00010011'   # 19

                    # Organizes the binary/integer formats for the data structures.
                    nN_A = int(Non_transcribed_A, 2)
                    nN_C = int(Non_transcribed_C, 2)
                    nN_G = int(Non_transcribed_G, 2)
                    nN_T = int(Non_transcribed_T, 2)
                    nT_A = int(Transcribed_A, 2)
                    nT_C = int(Transcribed_C, 2)
                    nT_G = int(Transcribed_G, 2)
                    nT_T = int(Transcribed_T, 2)
                    nU_A = int(Untranscribed_A, 2)
                    nU_C = int(Untranscribed_C, 2)
                    nU_G = int(Untranscribed_G, 2)
                    nU_T = int(Untranscribed_T, 2)
                    nB_A = int(BiDirectional_A, 2)
                    nB_C = int(BiDirectional_C, 2)
                    nB_G = int(BiDirectional_G, 2)
                    nB_T = int(BiDirectional_T, 2)
                    
                    nN_N = int(Non_transcribed_N, 2)
                    nT_N = int(Transcribed_N, 2)
                    nU_N = int(Untranscribed_N, 2)
                    nB_N = int(BiDirectional_N, 2)

                    byte_ref = {'N_A':bytes([nN_A]), 'N_C':bytes([nN_C]), 'N_G':bytes([nN_G]), 'N_T':bytes([nN_T]),
                                'T_A':bytes([nT_A]), 'T_C':bytes([nT_C]), 'T_G':bytes([nT_G]), 'T_T':bytes([nT_T]),
                                'U_A':bytes([nU_A]), 'U_C':bytes([nU_C]), 'U_G':bytes([nU_G]), 'U_T':bytes([nU_T]),
                                'B_A':bytes([nB_A]), 'B_C':bytes([nB_C]), 'B_G':bytes([nB_G]), 'B_T':bytes([nB_T]),
                                'N_N':bytes([nN_N]), 'T_N':bytes([nT_N]), 'U_N':bytes([nU_N]), 'B_N':bytes([nB_N])}

                    stringTest = ''


                    l = 0
                    line_count = 1
                    pointer = 0
                    with open(transcript_path + files, 'r') as f:
                        all_lines2 = f.readlines()
                        all_lines = all_lines2[:]
            #            all_lines = all_lines2[1:]

                        for lines in all_lines:
                            
                            # Handles the first line separately. 
                            if pointer == 0:
                                first_line = lines.split()
                                location = int(first_line[4]) -1
                                I_chrom = first_line[2][3:]
                                I_T_start = int(first_line[4]) -1
                                I_T_end = int(first_line[5]) - 1
                                I_strand = first_line[3]
                                
                                # Saves Non-transcribed data up until
                                # the first transcript.
                                for i in range(0, location, 1):
                                    nuc = chrom_string[i]
                                    outFile.write(byte_ref['N_' + nuc])
                                    l += 1
                                pointer = 1

                            # Handles all other lines after the first.
                            else:
                                first_line = lines.split()
                                I_chrom = first_line[2][3:]
                                I_T_start = int(first_line[4]) - 1
                                I_T_end = int(first_line[5]) - 1
                                I_strand = first_line[3]

                                # Saves Non-transcribed data up until the 
                                # next transcript.
                                if I_T_start > location:
                                    for i in range(location, I_T_start, 1):
                                        nuc = chrom_string[i]
                                        outFile.write(byte_ref['N_' + nuc])
                                        l += 1
                                    location = I_T_start
                                
                            # Reads the subsequent line to look for bi-directional
                            # transcription sites.
                            for line2 in all_lines[line_count:]:
                                next_line = line2.split()
                                c_chrom = next_line[2][3:]
                                c_start = int(next_line[4]) - 1
                                c_end= int(next_line[5]) - 1
                                c_strand = next_line[3]
                                
                                # Breaks to the next line if the two transcripts 
                                # don't overlap.
                                if c_start > I_T_end:
                                    break

                                # Checks if the transcripts are on opposite strands
                                # if they overlap.    
                                else:
                                    if c_strand != I_strand:
                                        bi_start = c_start
                                        if c_end > I_T_end:
                                            bi_end = I_T_end
                                        else:
                                            bi_end = c_end

                                    # Saves Un/Transcribed data up until the start of
                                    # the bi-directional transcription.
                                    if bi_start != 0 and bi_start > location:
                                        for i in range (location, bi_start, 1):
                                            if I_strand == '-1':
                                                nuc = chrom_string[i]
                                                outFile.write(byte_ref['T_' + nuc])
                                                l += 1
                                            else:
                                                nuc = chrom_string[i]
                                                outFile.write(byte_ref['U_' + nuc])
                                                l += 1
                                        location = bi_start

                                        # Saves Bi-directional data for the length
                                        # of the opposing transcripts overlap
                                        for i in range (location, bi_end, 1):
                                            nuc = chrom_string[i]
                                            outFile.write(byte_ref['B_' + nuc])
                                            l += 1
                                        location = bi_end
                                    bi_start = 0
                                    bi_end = 0

                            # Saves Un/Transcribed data up to the end of the current
                            # transcript if data has not already been saved for these bases.
                            if I_T_end > location:
                                for i in range(location, I_T_end, 1):
                                    if I_strand == '-1':
                                        nuc = chrom_string[i]
                                        outFile.write(byte_ref['T_' + nuc])
                                        l += 1
                                    else:
                                        nuc = chrom_string[i]
                                        outFile.write(byte_ref['U_' + nuc])
                                        l +=1
                                location = I_T_end
                            line_count += 1

                    # Save Non-transcribed data for the remainder of the chromosome after
                    # the final transcript is analyzed         
                    if location < chrom_length:
                        for i in range(location, chrom_length,1):
                            nuc = chrom_string[i]
                            outFile.write(byte_ref['N_' + nuc]) 
                            l += 1        
                        
                    outFile.close()
                    if first:
                        print("The transcript reference file has been created for Chromosome: " +chrom + " (" + str(z) + "/" + str(chrom_total) + ")")
                        first = False
                    else:
                        print("                                                               " +chrom + " (" + str(z) + "/" + str(chrom_total) + ")")
                    z += 1
            except:
                print(chrom + " has been skipped. This chromosome is not supported.")

    end_time = time.time()
    print("Transcript files created.\n Job took: ", end_time-start_time, " seconds")

def main ():
    parser = argparse.ArgumentParser(description="Provide the necessary arguments to create the transcriptional strand bias reference strings.")
    parser.add_argument("--genome", "-g",help="Provide a reference genome. (ex: GRCh37, GRCh38, mm10)")
    args=parser.parse_args()
    genome = args.genome

    script_dir = os.getcwd()
    ref_dir = re.sub('\/scripts$', '', script_dir)

    chromosome_string_path = ref_dir + "/references/chromosomes/chrom_string/" + genome + "/"
    transcript_path = ref_dir + "/references/chromosomes/transcripts/" + genome + "/"
    output_path = ref_dir + "/references/chromosomes/tsb/" + genome + "/"
    if os.path.exists(output_path) == False:
        os.makedirs(output_path)

    save_tsb(chromosome_string_path, transcript_path, output_path)

if __name__ == '__main__':
    main()





#!/usr/bin/env python3
 
#Author: Erik Bergstrom

#Contact: ebergstr@eng.ucsd.edu

import os
import sys
import re



def convertVCF (project, vcf_path, genome, output_path):
	files = os.listdir(vcf_path)
	first_indel = True
	first_SNV = True

	snv = False
	indel = False
	first_incorrect_file = True
	for file in files:
		file_name = file.split(".")
		sample = file_name[0]
		if file == '.DS_Store':
			continue
		with open (vcf_path + file) as f:
			for lines in f:
				if lines[0] == "#":
					continue
				else:
					try:
						line = lines.strip().split('\t')
						chrom = line[0]
						if len(chrom) > 2:
							chrom = chrom[3:]
						start = line[1]
						ref = line[3]
						mut = line[4]
						int(start)
					except:
						if first_incorrect_file:
							print("The given input files do not appear to be in the correct vcf format. Skipping this file: ", file)
							first_incorrect_file = False
						continue

					if len(ref) == 1 and len(mut) == 1 and ref != "-" and ref != "-":
						snv = True
						if first_SNV:
							outputFile = output_path + "SNV/" + project + ".genome"
							os.system("rm -f " + outputFile)
							if not os.path.exists(output_path + "SNV/"):
								os.mkdir(output_path + 'SNV/')
							out_snv = open(outputFile, "w") 
							first_SNV = False

						print("\t".join([project, sample, ".", genome, "SNV", chrom, start, start, ref, mut, "SOMATIC"]), file=out_snv)

					else:
						indel = True
						if first_indel:
							outputFile = output_path + "INDEL/" + project + ".genome"
							os.system("rm -f " + outputFile)
							if not os.path.exists(output_path + "INDEL/"):
								os.mkdir(output_path + "INDEL/")
							out_indel = open(outputFile, "w") 
							first_indel = False

						print("\t".join([project, sample, ".", genome, "INDEL", chrom, start, start, ref, mut, "SOMATIC"]), file=out_indel)

		first_incorrect_file = True


	out_snv.close()
	out_indel.close()
	return(snv, indel)


def convertTxt (project, vcf_path, genome, output_path):
	files = os.listdir(vcf_path)
	first_indel = True
	first_SNV = True	

	snv = False
	indel = False
	first_incorrect_file = True
	for file in files:
		if file == '.DS_Store':
			continue
		with open (vcf_path + file) as f:
			for lines in f:
				try:
					line = lines.strip().split('\t')
					sample = line[1]
					genome = line[3]
					chrom = line[5]
					if len(chrom) > 2:
						chrom = chrom[3:]
					start = line[6]
					end = line[7]
					ref = line[8]
					mut = line[9]
					int(start)
					int(end)
				except:
					if first_incorrect_file:
						print("The given input files do not appear to be in the correct simple text format. Skipping this file: ", file)
						first_incorrect_file = False
					continue

				if len(ref) == 1 and len(mut) == 1 and ref != "-" and ref != "-":
					snv = True
					if first_SNV:
						outputFile = output_path + "SNV/" + project + ".genome"
						os.system("rm -f " + outputFile)
						if not os.path.exists(output_path + "SNV/"):
							os.mkdir(output_path + 'SNV/')
						out_snv = open(outputFile, "w") 
						first_SNV = False

					print("\t".join([project, sample, ".", genome, "SNV", chrom, start, start, ref, mut, "SOMATIC"]), file=out_snv)
				
				else:
					indel = True
					if first_indel:
						outputFile = output_path + "INDEL/" + project + ".genome"
						os.system("rm -f " + outputFile)
						if not os.path.exists(output_path + "INDEL/"):
							os.mkdir(output_path + "INDEL/")
						out_indel = open(outputFile, "w") 
						first_indel = False

					print("\t".join([project, sample, ".", genome, "INDEL", chrom, start, start, ref, mut, "SOMATIC"]), file=out_indel)

		first_incorrect_file = True

	out_snv.close()
	out_indel.close()
	return(snv, indel)

def convertMAF (project, vcf_path, genome, output_path):
	files = os.listdir(vcf_path)
	first_indel = True
	first_SNV = True	

	snv = False
	indel = False
	first_incorrect_file = True
	for file in files:
		if file == '.DS_Store':
			continue
		name = file.split(".")
		sample = name[0]
		with open (vcf_path + file) as f:
			for lines in f:
				try:
					line = lines.strip().split('\t')
					genome = line[3]
					chrom = line[4]
					if len(chrom) > 2:
						chrom = chrom[3:]
					start = line[5]
					end = line[6]
					ref = line[10]
					mut = line[47]
					int(start)
					int(end)
				except:
					if first_incorrect_file:
						print("The given input files do not appear to be in the correct MAF format. Skipping this file: ", file)
						first_incorrect_file = False
					continue
				if len(ref) == 1 and len(mut) == 1 and ref != "-" and ref != "-":
					snv = True
					if first_SNV:
						outputFile = output_path + "SNV/" + project + ".genome"
						os.system("rm -f " + outputFile)
						if not os.path.exists(output_path + "SNV/"):
							os.mkdir(output_path + 'SNV/')
						out_snv = open(outputFile, "w") 
						first_SNV = False

					print("\t".join([project, sample, ".", genome, "SNV", chrom, start, start, ref, mut, "SOMATIC"]), file=out_snv)
				
				else:
					indel = True
					if first_indel:
						outputFile = output_path + "INDEL/" + project + ".genome"
						os.system("rm -f " + outputFile)
						if not os.path.exists(output_path + "INDEL/"):
							os.mkdir(output_path + "INDEL/")
						out_indel = open(outputFile, "w") 
						first_indel = False

					print("\t".join([project, sample, ".", genome, "INDEL", chrom, start, start, ref, mut, "SOMATIC"]), file=out_indel)

		first_incorrect_file = True

	out_snv.close()
	out_indel.close()
	return(snv, indel)


def convertICGC (project, vcf_path, genome, output_path):
	files = os.listdir(vcf_path)
	first_indel = True
	first_SNV = True	

	snv = False
	indel = False
	first_incorrect_file = True
	for file in files:
		if file == '.DS_Store':
			continue
		with open (vcf_path + file) as f:
			for lines in f:
				try:
					line = lines.strip().split('\t')
					sample = line[1]
					icgc_sample_id = line[4]
					chrom = line[8]
					if len(chrom) > 2:
						chrom = chrom[3:]
					start = int(line[9])
					end = line[10]
					genome = line[12]
					ref = line[15]
					mut = line[16]
					if ref == '-':
						mut = '-' + mut
					elif mut == '-':
						start -= 1
						ref = '-' + ref
					int(start)
					int(end)
				except:
					if first_incorrect_file:
						print("The given input files do not appear to be in the correct ICGC format.")
						first_incorrect_file = False
					continue

				if len(ref) == 1 and len(mut) == 1 and ref != "-" and ref != "-":
					snv = True
					if first_SNV:
						outputFile = output_path + "SNV/" + project + ".genome"
						os.system("rm -f " + outputFile)
						if not os.path.exists(output_path + "SNV/"):
							os.mkdir(output_path + 'SNV/')
						out_snv = open(outputFile, "w") 
						first_SNV = False

					print("\t".join([project, sample, ".", genome, "SNV", chrom, start, start, ref, mut, "SOMATIC"]), file=out_snv)
				
				else:
					indel = True
					if first_indel:
						outputFile = output_path + "INDEL/" + project + ".genome"
						os.system("rm -f " + outputFile)
						if not os.path.exists(output_path + "INDEL/"):
							os.mkdir(output_path + "INDEL/")
						out_indel = open(outputFile, "w") 
						first_indel = False

					print("\t".join([project, sample, ".", genome, "INDEL", chrom, start, start, ref, mut, "SOMATIC"]), file=out_indel)

		first_incorrect_file = True

	out_snv.close()
	out_indel.close()
	return(snv, indel)



